//////////////////////////////////////////////////////////
//
//  Whupsey - A Whups Client Library in C
//
//  21.11.2002
//

#ifndef WHUPSEY_H
#define WHUPSEY_H

#include <xmlrpc.h>
#include <xmlrpc_client.h>

typedef struct Whupsey              Whupsey;
typedef struct WhupseyState         WhupseyState;
typedef struct WhupseyPriority      WhupseyPriority;
typedef struct WhupseyType          WhupseyType;
typedef struct WhupseyVersion       WhupseyVersion;
typedef struct WhupseySubject       WhupseySubject;
typedef struct WhupseyModule        WhupseyModule;
typedef struct WhupseyAttributeDesc WhupseyAttributeDesc;
typedef struct WhupseyAttribute     WhupseyAttribute;
typedef struct WhupseyTicket        WhupseyTicket;

typedef enum WhupseyResult        WhupseyResult;
typedef enum WhupseyStateCategory WhupseyStateCategory;

enum WhupseyResult
{
    WHUPSEY_OK,
    WHUPSEY_NOMEMORY,
    WHUPSEY_RPCFAULT,
    WHUPSEY_BADARG,
};

enum WhupseyStateCategory
{
    WHUPSEY_UNCONFIRMED = 1,
    WHUPSEY_NEW,
    WHUPSEY_ASSIGNED,
    WHUPSEY_RESOLVED,
};

struct WhupseyState
{
    unsigned int id;

    char *name;
    char *description;

    WhupseyStateCategory  category;
};

struct WhupseyPriority
{
    unsigned int id;

    char *name;
    char *description;
};

struct WhupseyType
{
    unsigned int id;

    char *name;
    char *description;

    unsigned int  nstates;
    WhupseyState *states;

    int npriorities;
    WhupseyPriority *priorities;

    int nattributes;
    WhupseyAttributeDesc *attributes;
};

struct WhupseyVersion
{
    unsigned int id;

    char *name;
    char *description;
};

struct WhupseySubject
{
    unsigned int id;
    char *name;
};

struct WhupseyModule
{
    unsigned int id;
    unsigned int has_versions;
    unsigned int has_subjects;

    char *name;
    char *description;

    int ntypes;
    WhupseyType **types;

    int nversions;
    WhupseyVersion *versions;

    int nsubjects;
    WhupseySubject *subjects;
};

struct WhupseyAttributeDesc
{
    unsigned int id;

    char *name;
    char *description;
};

struct WhupseyAttribute
{
    const WhupseyAttributeDesc *descriptor;
    char *value;
};

struct WhupseyTicket
{
    unsigned int  id;
    unsigned int  type_id;
    unsigned int  state_id;
    unsigned int  priority_id;
    unsigned int  module_id;
    unsigned int  version_id;
    unsigned int  comment_id;
    unsigned long timestamp;

    char *requester;
    char *owner;

    char *summary;
    char *comment;

    // nattributes is -1 if this ticket was fetched from the server but
    // the attributes weren't, ie. we don't know how many attributes there are
    int nattributes;
    WhupseyAttribute *attributes;
};

extern WhupseyResult whupsey_create (Whupsey **result, const char *useragent, const char *version,
                                     const char *server, const char *user, const char *pass);

extern WhupseyResult whupsey_load_schema (Whupsey *w);

extern void whupsey_free (Whupsey *w);

extern int         whupsey_get_fault_code   (Whupsey *w);
extern const char *whupsey_get_fault_string (Whupsey *w);

extern const WhupseyType *whupsey_find_type_by_id   (Whupsey *w, unsigned int id);
extern const WhupseyType *whupsey_find_type_by_name (Whupsey *w, const char *name);

extern const WhupseyAttributeDesc *whupsey_find_attribute_desc_by_id   (Whupsey *w, WhupseyType *type, unsigned int id);
extern const WhupseyAttributeDesc *whupsey_find_attribute_desc_by_name (Whupsey *w, WhupseyType *type, const char *name);

extern void whupsey_get_types   (Whupsey *w, const WhupseyType   **types,   int *ntypes);
extern void whupsey_get_modules (Whupsey *w, const WhupseyModule **modules, int *nmodules);

extern WhupseyResult whupsey_get_assigned_tickets  (Whupsey *w, int **result, int *count);
extern WhupseyResult whupsey_get_requested_tickets (Whupsey *w, int **result, int *count);

extern WhupseyResult whupsey_add_ticket (Whupsey *w, WhupseyTicket *ticket);

// This API will probably change very soon to take WhupseyTicket parameter and update it
// with the new attributes
extern WhupseyResult whupsey_set_ticket_attributes (Whupsey *w, unsigned int ticket_id, const char *comment,
                                                    WhupseyAttribute *attribs, int nattribs);

#endif

